//
// Created by trotfunky on 06/06/19.
//

#ifndef SRC_ENTITY_H
#define SRC_ENTITY_H

#include <SFML/Graphics.hpp>
#include <pugixml.hpp>
#include <cinttypes>

#include "Utils.h"


enum class EntityType
{
    Citizen,
    Significant,
    House,
    Car,
};

enum class State
{
    Moving,
    Fleeing,
    Waiting,
    Idle,
};

enum class Orientation
{
    North = 0,
    East = 90,
    South = 180,
    West = 270,
};


class Entity
{
public:
    /// x,y, width and height are in grid coordinates
    Entity(int x, int y, EntityType type, sf::Texture* texture, int width = 1, int height = 1);
    explicit Entity(const pugi::xml_node& entityNode, sf::Texture* texture);

    void move(Orientation orientation);
    void update();

    const sf::RectangleShape& getShape() const;
    const State getState() const;
    const pro_maat::GridPos getPosition() const;
    // Don't like it : iterates over every square every tick
    const std::vector<pro_maat::GridPos> getOccupiedSquares() const;


    /// Position of the target of the current action on the map
    pro_maat::GridPos target;
private:
    static const std::map<std::string,EntityType> entityTypeLookup;

    EntityType type;

    // As it contains position, size and orientation, we do not need anything more
    sf::RectangleShape shape;

    State currentState;
    /// Used with rules : last to update has priority
    State nextState;

    /// Used with rules : last to update has priority
    pro_maat::GridPos nextTarget;
};


#endif //SRC_ENTITY_H
